"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var x_logger_1 = require("@empathyco/x-logger");
var StorageService = /** @class */ (function () {
    function StorageService(storage, prefix) {
        if (storage === void 0) { storage = localStorage; }
        if (prefix === void 0) { prefix = 'empathy'; }
        this.storage = storage;
        this.prefix = prefix;
        this.logger = x_logger_1.logger.child("[StorageService][" + prefix + "]");
    }
    StorageService.prototype.setItem = function (key, item, ttlInMs) {
        if (item === undefined) {
            this.logger.warn("Tried to store an undefined object with key " + key);
        }
        else {
            var prefixedKey = this.prefixKey(key);
            var expirableItem = this.createExpirableItem(item, ttlInMs);
            var serializedItem = JSON.stringify(expirableItem);
            this.storage.setItem(prefixedKey, serializedItem);
        }
    };
    StorageService.prototype.getItem = function (key) {
        this.removeExpiredItems();
        var prefixedKey = this.prefixKey(key);
        var serializedItem = this.storage.getItem(prefixedKey);
        if (serializedItem) {
            var item = JSON.parse(serializedItem);
            return this.getItemValue(item);
        }
        return null;
    };
    StorageService.prototype.removeItem = function (key) {
        var item = this.getItem(key);
        var prefixedKey = this.prefixKey(key);
        this.storage.removeItem(prefixedKey);
        return item;
    };
    StorageService.prototype.clear = function () {
        var _this = this;
        return this.getOwnKeys().reduce(function (removedCount, key) {
            _this.storage.removeItem(key);
            return ++removedCount;
        }, 0);
    };
    StorageService.prototype.prefixKey = function (key) {
        return this.prefix + "-" + key;
    };
    StorageService.prototype.createExpirableItem = function (item, ttlInMs) {
        return tslib_1.__assign(tslib_1.__assign({}, (!!ttlInMs && { ttl: ttlInMs + this.currentTimestamp() })), { value: item });
    };
    StorageService.prototype.currentTimestamp = function () {
        return Date.now();
    };
    StorageService.prototype.getItemValue = function (item) {
        return item.value;
    };
    StorageService.prototype.getOwnKeys = function () {
        var _this = this;
        return Object.keys(this.storage).filter(function (key) { return key.startsWith(_this.prefix + "-"); });
    };
    StorageService.prototype.removeExpiredItems = function () {
        var _this = this;
        this.getOwnKeys().forEach(function (key) {
            var serializedItem = _this.storage.getItem(key);
            if (serializedItem) {
                try {
                    var item = JSON.parse(serializedItem);
                    if (item.ttl && item.ttl <= _this.currentTimestamp()) {
                        _this.storage.removeItem(key);
                    }
                }
                catch (_a) {
                    _this.logger.warn(
                    // eslint-disable-next-line max-len
                    "Item for key " + key + " has been removed from storage because it had an invalid JSON value: \"" + serializedItem + "\"");
                    _this.storage.removeItem(key);
                }
            }
        });
    };
    return StorageService;
}());
exports.StorageService = StorageService;
//# sourceMappingURL=storage-service.js.map